import pytz
from datetime import datetime
from django.utils import timezone
from math import log

# uses a version of reddit score algorithm
# https://medium.com/hacking-and-gonzo/how-reddit-ranking-algorithms-work-ef111e33d0d9#.aef67efq1


def question_score(question):
    creation_date = question.pub_date
    score = question.total_points
    answers_positive_points = list(
        question.answer_set.all().values_list(
            'answervote__value', flat=True)).count(True)
    answers_negative_points = list(
        question.answer_set.all().values_list(
            'answervote__value', flat=True)).count(False)
    score = score * 2 + answers_positive_points - answers_negative_points
    reference_date = pytz.timezone(
        timezone.get_default_timezone_name()).localize(datetime(1970, 1, 1))
    difference = creation_date - reference_date
    difference_seconds = difference.days * 86400 + difference.seconds +\
        (float(difference.microseconds) / 1000000)
    order = log(max(abs(score), 1), 10)
    sign = 1 if score > 0 else -1 if score < 0 else 0
    seconds = difference_seconds - 1134028003
    return round(sign * order + seconds / 45000, 7)



from django.db import models
from django.conf import settings
from django_markdown.models import MarkdownField

from taggit.managers import TaggableManager
from annoying.fields import AutoOneToOneField


class UserQAProfile(models.Model):
    user = AutoOneToOneField(settings.AUTH_USER_MODEL, primary_key=True)
    points = models.IntegerField(default=0)
    # The additional attributes we wish to include.
    website = models.URLField(blank=True)

    def __str__(self):  # pragma: no cover
        return self.user.username


class Question(models.Model):
    title = models.CharField(max_length=200, blank=False)
    description = MarkdownField()
    pub_date = models.DateTimeField('date published', auto_now_add=True)
    tags = TaggableManager()
    reward = models.IntegerField(default=0)
    views = models.IntegerField(default=0)
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    closed = models.BooleanField(default=False)
    positive_votes = models.IntegerField(default=0)
    negative_votes = models.IntegerField(default=0)
    total_points = models.IntegerField(default=0)

    def save(self, *args, **kwargs):
        self.total_points = self.positive_votes - self.negative_votes
        super(Question, self).save(*args, **kwargs)

    def __str__(self):
        return self.title


class Answer(models.Model):
    question = models.ForeignKey(Question)
    answer_text = MarkdownField()
    pub_date = models.DateTimeField('date published', auto_now_add=True)
    updated = models.DateTimeField('date updated', auto_now=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    answer = models.BooleanField(default=False)
    positive_votes = models.IntegerField(default=0)
    negative_votes = models.IntegerField(default=0)
    total_points = models.IntegerField(default=0)

    def save(self, *args, **kwargs):
        self.total_points = self.positive_votes - self.negative_votes
        super(Answer, self).save(*args, **kwargs)

    def __str__(self):  # pragma: no cover
        return self.answer_text

    class Meta:
        ordering = ['answer', '-pub_date']


class VoteParent(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    value = models.BooleanField(default=True)

    class Meta:
        abstract = True


class AnswerVote(VoteParent):
    answer = models.ForeignKey(Answer)

    class Meta:
        unique_together = (('user', 'answer'),)


class QuestionVote(VoteParent):
    question = models.ForeignKey(Question)

    class Meta:
        unique_together = (('user', 'question'),)


class BaseComment(models.Model):
    pub_date = models.DateTimeField('date published', auto_now_add=True)
    user = models.ForeignKey(settings.AUTH_USER_MODEL)

    class Meta:
        abstract = True

    def __str__(self):  # pragma: no cover
        return self.comment_text


class AnswerComment(BaseComment):
    comment_text = MarkdownField()
    answer = models.ForeignKey(Answer)


class QuestionComment(BaseComment):
    comment_text = models.CharField(max_length=250)
    question = models.ForeignKey(Question)



from django.test import TestCase, Client
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.core.exceptions import ValidationError
from qa.models import (Question, Answer, QuestionComment, QuestionVote,
                       AnswerVote)
from qa.mixins import LoginRequired
from qa.views import CreateQuestionView, CreateAnswerView


class TestViews(TestCase):
    """
    Includes tests for all the functionality
    associated with Views
    """
    def setUp(self):
        self.client = Client()
        self.user = get_user_model().objects.create_user(
            username='test_user',
            email='test@swapps.co',
            password='top_secret'
        )
        self.client.login(username='test_user', password='top_secret')

    def test_create_question_login(self):
        """
        CreateQuestionView should require login.
        Assertion can be made just by checking that the correct mixin
        is a superclass.
        """
        self.assertTrue(issubclass(CreateQuestionView, LoginRequired))

    def test_create_answer_login(self):
        """
        CreateAnswerView should require login.
        """
        self.assertTrue(issubclass(CreateAnswerView, LoginRequired))

    def test_create_question_view_one(self):
        """
        CreateQuestionView should create a new question object.
        """
        title = 'This is my question'
        current_question_count = Question.objects.count()
        response = self.client.post(
            reverse('qa_create_question'),
            {'title': title, 'description': 'babla', 'tags': 'test tag'})
        self.assertEqual(response.status_code, 302)
        new_question = Question.objects.first()
        self.assertEqual(new_question.title, title)
        self.assertEqual(Question.objects.count(),
                         current_question_count + 1)

    def test_create_question_view_two(self):
        """
        CreateAnswerView should create a new question object bound to the
        given question.
        """
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        current_answer_count = Answer.objects.filter(question=question).count()
        response = self.client.post(
            reverse('qa_create_answer', kwargs={'question_id': question.pk}),
            {'question': question, 'answer_text': 'some_text_here'})
        self.assertEqual(response.status_code, 302)
        new_answer = Answer.objects.first()
        self.assertEqual(new_answer.question, question)
        self.assertEqual(
            Answer.objects.count(), current_answer_count + 1)

# CreateQuestionCommentView

    def test_create_question_comment_view(self):
        """
        CreateQuestionCommentView should create a new question comment object
        bound to the
        given question.
        """
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        current_comment_question_count = QuestionComment.objects.filter(
            question=question).count()
        response = self.client.post(
            reverse('qa_create_question_comment',
                    kwargs={'question_id': question.pk}),
            {'comment_text': 'some_text_here'})
        self.assertEqual(response.status_code, 302)
        new_comment = QuestionComment.objects.first()
        self.assertEqual(new_comment.question, question)
        self.assertEqual(QuestionComment.objects.count(),
                         current_comment_question_count + 1)

    def test_user_cannot_upvote_own_questions(self):
        """
        When some user upvotes a question, and it is his own question
        an error should be raised, because that's not allowed.
        """
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        with self.assertRaises(ValidationError):
            self.client.post(reverse(
                'qa_question_vote', kwargs={'object_id': question.id}),
                data={'upvote': 'on'})

    def test_can_upvote_question(self):
        """
        When i upvote a question, the question field gets updated
        and a new instance of QuestionVote is created.
        Shares same base class as answer votes.
        """
        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        previous_votes = question.total_points
        previous_vote_instances = QuestionVote.objects.count()
        response = self.client.post(reverse(
            'qa_question_vote', kwargs={'object_id': question.id}),
            data={'upvote': 'on'})
        self.assertEqual(response.status_code, 302)
        question.refresh_from_db()
        self.assertEqual(previous_votes + 1, question.total_points)
        self.assertEqual(previous_vote_instances + 1,
                         QuestionVote.objects.count())

    def test_can_downvote_answer(self):
        """
        When i downvote an answer, the answer field gets updated
        and a new instance of AnswerVote is created.
        """
        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        answer = Answer.objects.create(
            answer_text='a title', user=user, question=question)
        previous_votes = question.total_points
        previous_vote_instances = AnswerVote.objects.count()
        response = self.client.post(reverse('qa_answer_vote',
                                    kwargs={'object_id': answer.id}))
        self.assertEqual(response.status_code, 302)
        answer.refresh_from_db()
        self.assertEqual(previous_votes - 1, answer.total_points)
        self.assertEqual(answer.negative_votes, 1)
        self.assertEqual(previous_vote_instances + 1,
                         AnswerVote.objects.count())

    def test_upvote_question_deletes_instance(self):
        """
        When a user upvotes a question that was already upvoted, the vote
        is reverted. This means that the vote count goes down and the
        vote instance is removed.
        """
        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        QuestionVote.objects.create(user=self.user, question=question,
                                    value=True)
        previous_vote_instances = QuestionVote.objects.count()
        previous_votes = question.total_points
        self.client.post(reverse('qa_question_vote',
                         kwargs={'object_id': question.id}),
                         data={'upvote': 'on'})
        self.assertEqual(previous_vote_instances - 1,
                         QuestionVote.objects.count())
        question.refresh_from_db()
        self.assertEqual(previous_votes - 1, question.total_points)

    def test_switching_vote_updates_correctly(self):
        """
        If i downvote an already upvoted question, the count should
        shift downwards by 2 because i am not only retiring my upvote, I am
        adding a negative vote too. The vote instance should be updated too.
        """
        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        question_vote = QuestionVote.objects.create(user=self.user,
                                                    question=question,
                                                    value=True)
        previous_vote_instances = QuestionVote.objects.count()
        previous_votes = question.total_points
        previous_question_vote = question_vote.value
        self.client.post(reverse('qa_question_vote',
                         kwargs={'object_id': question.id}))
        self.assertEqual(previous_vote_instances, QuestionVote.objects.count())
        question.refresh_from_db()
        question_vote.refresh_from_db()
        self.assertEqual(previous_votes - 2, question.total_points)
        self.assertNotEqual(previous_question_vote, question_vote.value)

# AnswerQuestionView

    def test_can_mark_answer_as_satisfying_answer(self):
        """
        The owner of the question should be able to mark an answer
        as the satisfying one. The question should be kept open
        """

        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        answer = Answer.objects.create(
            answer_text='a title', user=user, question=question)
        response = self.client.post(reverse('qa_answer_question',
                                    kwargs={'answer_id': answer.id}))
        answer.refresh_from_db()
        question.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertFalse(question.closed)
        self.assertTrue(answer.answer)

    def test_can_provide_next_url_when_marking_satisfying_answer(self):
        """
        If an url is provided at the post request, the view should
        redirect there.
        """

        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        answer = Answer.objects.create(
            answer_text='a title', user=user, question=question)
        response = self.client.post(
            reverse('qa_answer_question',
                    kwargs={'answer_id': answer.id}),
            data={'next': reverse('qa_create_question')})
        answer.refresh_from_db()
        question.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, reverse('qa_create_question'))

    def test_not_owner_cannot__mark_answer_as_satisfying_answer(self):
        """
        Any user that is not the owner of the question should not be able
        to mark an user as the satisfying one.
        """

        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        answer = Answer.objects.create(
            answer_text='a title', user=self.user, question=question)
        with self.assertRaises(ValidationError):
            self.client.post(reverse('qa_answer_question',
                             kwargs={'answer_id': answer.id}))
            answer.refresh_from_db()
            self.assertFalse(answer.answer)

# CloseQuestionView

    def test_can_mark_close_question(self):
        """
        The owner of the question should be able to
        close it
        """
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        response = self.client.post(reverse('qa_close_question',
                                    kwargs={'question_id': question.id}))
        question.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertTrue(question.closed)

    def test_can_provide_next_url_when_closing_question(self):
        """
        If an url is provided at the post request, the view should
        redirect there.
        """
        question = Question.objects.create(
            title='a title', description='bla', user=self.user)
        response = self.client.post(
            reverse('qa_close_question',
                    kwargs={'question_id': question.id}),
            data={'next': reverse('qa_create_question')})
        question.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, reverse('qa_create_question'))

    def test_not_owner_cannot_close_question(self):
        """
        Any user that is not the owner of the question should not be able
        to close question.
        """
        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        with self.assertRaises(ValidationError):
            self.client.post(reverse('qa_close_question',
                             kwargs={'question_id': question.id}))
            self.assertFalse(question.closed)


# QuestionIndexView

    def test_question_index_returns_all_questions(self):
        """
        QuestionIndexView should return all questions,
        questions with no answers and the questions with
        points
        """
        title = 'This is my question'
        self.client.post(
            reverse('qa_create_question'),
            {'title': title, 'description': 'babla', 'tags': 'test tag'})
        response = self.client.get(reverse('qa_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            list(response.context['questions']),
            list(Question.objects.all()))
        self.assertEqual(
            list(response.context['noans']),
            list(Question.objects.all()))
        self.assertEqual(
            list(response.context['reward']), [])

    def test_question_search_returns_related_questions(self):
        """
        QuestionsSearchView should return questions
        containing the search term
        """
        self.client.post(
            reverse('qa_create_question'),
            {'title': "first title",
             'description': 'first description',
             'tags': 'test tag'})
        self.client.post(
            reverse('qa_create_question'),
            {'title': "second title",
             'description': 'second description',
             'tags': 'test tag'})
        response = self.client.get(reverse('qa_search'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            len(response.context['questions']),
            len(Question.objects.all()))
        self.assertEqual(
            len(response.context['noans']),
            len(Question.objects.all()))
        self.assertEqual(
            len(response.context['reward']), 0)
        response = self.client.get(
            reverse('qa_search'), data={'word': 'first'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            len(response.context['questions']),
            len(Question.objects.filter(title="first title")))
        response = self.client.get(
            reverse('qa_search'), data={'word': 'second'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            len(response.context['questions']),
            len(Question.objects.filter(title="second title")))

# QuestionsByTagView

    def test_question_by_tag_returns_related_tag_questions(self):
        """
        QuestionsByTagView should return questions related
        with searched tag
        """
        self.client.post(
            reverse('qa_create_question'),
            {'title': "first title",
             'description': 'first description',
             'tags': 'tag'})
        self.client.post(
            reverse('qa_create_question'),
            {'title': "second title",
             'description': 'second description',
             'tags': 'test'})
        response = self.client.get(
            reverse('qa_tag', kwargs={'tag': 'tag'}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            len(response.context['questions']),
            len(Question.objects.filter(title="first title")))
        response = self.client.get(
            reverse('qa_tag', kwargs={'tag': 'test'}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            len(response.context['questions']),
            len(Question.objects.filter(title="second title")))

# UpdateQuestionView

    def test_updates_question_modify_question(self):
        """
        UpdateQuestionView updates the required question
        """
        self.client.post(
            reverse('qa_create_question'),
            {'title': "first title",
             'description': 'first description',
             'tags': 'tag'})
        question = Question.objects.latest('pub_date')
        question_title = question.title
        question_description = question.description
        response = self.client.post(
            reverse('qa_update_question',
                    kwargs={'question_id': question.id}),
            {'title': "second title",
             'description': 'second description',
             'tags': 'test'})
        question.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertNotEqual(question_title, question.title)
        self.assertNotEqual(question_description, question.description)


# UpdateAnswerView

    def test_updates_answer_modify_answer(self):
        """
        UpdateQuestionView updates the required answer
        """

        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        answer = Answer.objects.create(
            answer_text='a title', user=self.user, question=question)
        answer_text = answer.answer_text
        response = self.client.post(
            reverse('qa_update_answer',
                    kwargs={'answer_id': answer.id}),
            {'answer_text': 'a description'})
        answer.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertNotEqual(answer_text, answer.answer_text)

# CreateAnswerCommentView

    def test_create_answer_comment_create_comment_to_given_answer(self):
        """
        CreateAnswerCommentView creates comment to given answer
        """

        user = get_user_model().objects.create_user(username='user2',
                                                    password='top_secret')
        question = Question.objects.create(
            title='a title', description='bla', user=user)
        answer = Answer.objects.create(
            answer_text='a title', user=self.user, question=question)
        answer_comments = len(answer.answercomment_set.all())
        response = self.client.post(
            reverse('qa_create_answer_comment',
                    kwargs={'answer_id': answer.id}),
            {'comment_text': 'a description'})
        answer.refresh_from_db()
        self.assertEqual(response.status_code, 302)
        self.assertNotEqual(
            answer_comments, answer.answercomment_set.all())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_markdown.models
from django.conf import settings
import taggit.managers
import annoying.fields


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0002_auto_20150616_2121'),
        ('auth', '0006_require_contenttypes_0002'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('answer_text', django_markdown.models.MarkdownField()),
                ('votes', models.IntegerField(default=0)),
                ('pub_date', models.DateTimeField(auto_now_add=True, verbose_name=b'date published')),
            ],
        ),
        migrations.CreateModel(
            name='AnswerComment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('pub_date', models.DateTimeField(auto_now_add=True, verbose_name=b'date published')),
                ('comment_text', django_markdown.models.MarkdownField()),
                ('answer', models.ForeignKey(to='qa.Answer')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AnswerVote',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.BooleanField(default=True)),
                ('answer', models.ForeignKey(to='qa.Answer')),
            ],
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=200)),
                ('votes', models.IntegerField(default=0)),
                ('description', django_markdown.models.MarkdownField()),
                ('pub_date', models.DateTimeField(auto_now_add=True, verbose_name=b'date published')),
                ('reward', models.IntegerField(default=0)),
                ('views', models.IntegerField(default=0)),
                ('closed', models.BooleanField(default=False)),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags')),
            ],
        ),
        migrations.CreateModel(
            name='QuestionComment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('pub_date', models.DateTimeField(auto_now_add=True, verbose_name=b'date published')),
                ('comment_text', models.CharField(max_length=250)),
                ('question', models.ForeignKey(to='qa.Question')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='QuestionVote',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.BooleanField(default=True)),
                ('question', models.ForeignKey(to='qa.Question')),
            ],
        ),
        migrations.CreateModel(
            name='UserQAProfile',
            fields=[
                ('user', annoying.fields.AutoOneToOneField(primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('points', models.IntegerField(default=0)),
                ('website', models.URLField(blank=True)),
                ('picture', models.ImageField(upload_to=b'qa/static/profile_images', blank=True)),
            ],
        ),
        migrations.AddField(
            model_name='questionvote',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='questioncomment',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='question',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='answervote',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='answercomment',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='answer',
            name='question',
            field=models.ForeignKey(to='qa.Question'),
        ),
        migrations.AddField(
            model_name='answer',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='questionvote',
            unique_together=set([('user', 'question')]),
        ),
        migrations.AlterUniqueTogether(
            name='answervote',
            unique_together=set([('user', 'answer')]),
        ),
    ]



from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.views.generic import View
from django.utils.decorators import method_decorator


class LoginRequired(View):
    """
    Redirects to login if user is anonymous
    """
    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(LoginRequired, self).dispatch(*args, **kwargs)


class AuthorRequiredMixin(View):
    def dispatch(self, request, *args, **kwargs):
        obj = self.get_object()
        if obj.user != self.request.user:
            raise PermissionDenied

        return super(
            AuthorRequiredMixin, self).dispatch(request, *args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0004_answer_updated'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='negative_votes',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='answer',
            name='positive_votes',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='question',
            name='negative_votes',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='question',
            name='positive_votes',
            field=models.IntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-04-14 14:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0002_auto_20160412_1336'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='answer',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='answer',
            name='pub_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date published'),
        ),
        migrations.AlterField(
            model_name='answercomment',
            name='pub_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date published'),
        ),
        migrations.AlterField(
            model_name='question',
            name='pub_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date published'),
        ),
        migrations.AlterField(
            model_name='questioncomment',
            name='pub_date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='date published'),
        ),
        migrations.AlterField(
            model_name='userqaprofile',
            name='picture',
            field=models.ImageField(blank=True, upload_to='qa/static/profile_images'),
        ),
    ]



from django.contrib import admin
from qa.models import (Question, Answer, AnswerComment, AnswerVote,
                       QuestionComment)
from django_markdown.admin import MarkdownModelAdmin

admin.site.register(Question)
admin.site.register(Answer, MarkdownModelAdmin)
admin.site.register(AnswerComment)
admin.site.register(QuestionComment)
admin.site.register(AnswerVote)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-09-19 15:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0008_auto_20160719_0729'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='answer',
            options={'ordering': ['answer', '-pub_date']},
        ),
    ]



from django.utils import timezone
from django.core.exceptions import PermissionDenied
from django.test import TestCase, RequestFactory
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth import get_user_model
from django.conf import settings
from django.http import HttpResponse
from django.views.generic import View, DetailView
from ..mixins import LoginRequired, AuthorRequiredMixin
from ..models import Question


class SomeView(LoginRequired, View):

    def get(self, request):
        return HttpResponse('something')


class SomeAuthorRequiredView(AuthorRequiredMixin, DetailView):

    model = Question

    def get(self, request, pk):
        return HttpResponse('something')


class TestMixins(TestCase):
    """
    Tests functionalities for the mixins.
    Views that implement them should only test
    that they are subclasses of the given mixin.
    """
    def setUp(self):
        self.factory = RequestFactory()

    def test_login_required_redirects(self):
        """
        Anonymous users should be redirected to the LOGIN_URL
        """
        request = self.factory.get('some-random-place')
        request.user = AnonymousUser()
        response = SomeView.as_view()(request)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(settings.LOGIN_URL in response.url)

    def test_login_request_access(self):
        """
        Authenticated users should be able to access the view.
        """
        request = self.factory.get('some-random-place')
        request.user = get_user_model().objects.create_user(
            username='test_user',
            password='top_secret'
        )
        response = SomeView.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_author_required_allow_object_user(self):
        """
        The owner of the object should be able to access
        the view
        """
        request = self.factory.get('some-random-place')
        request.user = get_user_model().objects.create_user(
            username='test_user',
            password='top_secret'
        )
        question = Question.objects.create(
            title="Another Question",
            description="A not so long random text to fill this field",
            pub_date=timezone.datetime(2016, 1, 6, 0, 0, 0),
            reward=0,
            views=3,
            user=request.user,
            closed=False,
        )
        response = SomeAuthorRequiredView.as_view()(request, pk=question.pk)
        self.assertEqual(response.status_code, 200)

    def test_author_required_not_allow_not_object_user(self):
        """
        A different user than the object's owner should not
        be able to access the view, a PermissionDenied should
        be raised
        """
        request = self.factory.get('some-random-place')
        request.user = AnonymousUser()
        user = get_user_model().objects.create_user(
            username='test_user',
            password='top_secret'
        )
        question = Question.objects.create(
            title="Another Question",
            description="A not so long random text to fill this field",
            pub_date=timezone.datetime(2016, 1, 6, 0, 0, 0),
            reward=0,
            views=3,
            user=user,
            closed=False,
        )
        with self.assertRaises(PermissionDenied):
            SomeAuthorRequiredView.as_view()(
                request, pk=question.pk)



import operator
from functools import reduce
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.core.urlresolvers import reverse
from django.db.models import Count
from django.conf import settings
from django.utils.text import slugify
from django.utils.translation import ugettext as _
from django.views.generic import (CreateView, View, ListView, DetailView,
                                  UpdateView)
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db.models import Q
from qa.models import (UserQAProfile, Question, Answer, AnswerVote,
                       QuestionVote, AnswerComment, QuestionComment)
from .mixins import LoginRequired, AuthorRequiredMixin
from .utils import question_score
try:
    qa_messages = 'django.contrib.messages' in settings.INSTALLED_APPS and\
        settings.QA_MESSAGES
except AttributeError:
    qa_messages = False

if qa_messages:
    from django.contrib import messages


"""Dear maintainer:

Once you are done trying to 'optimize' this routine, and have realized what a
terrible mistake that was, please increment the following counter as a warning
to the next guy:

total_hours_wasted_here = 2
"""


class AnswerQuestionView(LoginRequired, View):
    """
    View to select an answer as the satisfying answer to the question,
    validating than the user who created que
    question is the only one allowed to make those changes.
    """
    model = Answer

    def post(self, request, answer_id):
        answer = get_object_or_404(self.model, pk=answer_id)
        if answer.question.user != request.user:
            raise ValidationError(
                "Sorry, you're not allowed to close this question.")
        else:
            # answer.question.closed = True
            # answer.question.save()
            answer.question.answer_set.update(answer=False)
            answer.answer = True
            answer.save()

        next_url = request.POST.get('next', None)
        if next_url is not None:
            return redirect(next_url)

        else:
            return redirect(reverse('qa_index'))


class CloseQuestionView(LoginRequired, View):
    """View to
    mark the question as closed, validating than the user who created que
    question is the only one allowed to make those changes.
    """
    model = Question

    def post(self, request, question_id):
        question = get_object_or_404(self.model, pk=question_id)
        if question.user != request.user:
            raise ValidationError(
                "Sorry, you're not allowed to close this question.")
        else:
            if not question.closed:
                question.closed = True
            else:
                question.closed = False
            question.save()
            # answer.answer = True
            # answer.save()

        next_url = request.POST.get('next', None)
        if next_url is not None:
            return redirect(next_url)

        else:
            return redirect(reverse('qa_index'))


class QuestionIndexView(ListView):
    """CBV to render the index view
    """
    model = Question
    paginate_by = 10
    context_object_name = 'questions'
    template_name = 'qa/index.html'
    ordering = '-pub_date'

    def get_context_data(self, *args, **kwargs):
        context = super(
            QuestionIndexView, self).get_context_data(*args, **kwargs)
        noans = Question.objects.order_by('-pub_date').filter(
            answer__isnull=True).select_related('user')\
            .annotate(num_answers=Count('answer', distinct=True),
                      num_question_comments=Count('questioncomment',
                      distinct=True))
        context['totalcount'] = Question.objects.count()
        context['anscount'] = Answer.objects.count()
        paginator = Paginator(noans, 10)
        page = self.request.GET.get('noans_page')
        context['active_tab'] = self.request.GET.get('active_tab', 'latest')
        tabs = ['latest', 'unans', 'reward']
        context['active_tab'] = 'latest' if context['active_tab'] not in\
            tabs else context['active_tab']
        try:
            noans = paginator.page(page)
        except PageNotAnInteger:
            noans = paginator.page(1)
        except EmptyPage:
            noans = paginator.page(paginator.num_pages)
        context['totalnoans'] = paginator.count
        context['noans'] = noans
        context['reward'] = Question.objects.order_by('-reward').filter(
            reward__gte=1)[:10]
        return context

    def get_queryset(self):
        queryset = super(QuestionIndexView, self).get_queryset()\
            .select_related('user')\
            .annotate(num_answers=Count('answer', distinct=True),
                      num_question_comments=Count('questioncomment',
                      distinct=True))
        return queryset


class QuestionsSearchView(QuestionIndexView):
    """
    Display a ListView page inherithed from the QuestionIndexView filtered by
    the search query and sorted by the different elements aggregated.
    """

    def get_queryset(self):
        result = super(QuestionsSearchView, self).get_queryset()
        query = self.request.GET.get('word', '')
        if query:
            query_list = query.split()
            result = result.filter(
                reduce(operator.and_,
                       (Q(title__icontains=q) for q in query_list)) |
                reduce(operator.and_,
                       (Q(description__icontains=q) for q in query_list))
            )

        return result

    def get_context_data(self, *args, **kwargs):
        context = super(
            QuestionsSearchView, self).get_context_data(*args, **kwargs)
        context['totalcount'] = Question.objects.count
        context['anscount'] = Answer.objects.count
        context['noans'] = Question.objects.order_by('-pub_date').filter(
            answer__isnull=True)[:10]
        context['reward'] = Question.objects.order_by('-reward').filter(
            reward__gte=1)[:10]
        return context


class QuestionsByTagView(ListView):
    """View to call all the questions clasiffied under one specific tag.
    """
    model = Question
    paginate_by = 10
    context_object_name = 'questions'
    template_name = 'qa/index.html'

    def get_queryset(self, **kwargs):
        return Question.objects.filter(tags__slug=self.kwargs['tag'])

    def get_context_data(self, *args, **kwargs):
        context = super(
            QuestionsByTagView, self).get_context_data(*args, **kwargs)
        context['totalcount'] = Question.objects.count
        context['anscount'] = Answer.objects.count
        context['noans'] = Question.objects.order_by('-pub_date').filter(
            tags__name__contains=self.kwargs['tag'], answer__isnull=True)[:10]
        context['reward'] = Question.objects.order_by('-reward').filter(
            tags__name__contains=self.kwargs['tag'],
            reward__gte=1)[:10]
        return context


class CreateQuestionView(LoginRequired, CreateView):
    """
    View to handle the creation of a new question
    """
    template_name = 'qa/create_question.html'
    model = Question
    fields = ['title', 'description', 'tags']
    message = _('Thank you! your question has been created.')

    def form_valid(self, form):
        """
        Create the required relation
        """
        form.instance.user = self.request.user
        return super(CreateQuestionView, self).form_valid(form)

    def get_success_url(self):
        if qa_messages:
            messages.success(
                self.request, self.message)
        return reverse('qa_index')


class UpdateQuestionView(LoginRequired, AuthorRequiredMixin, UpdateView):
    """
    Updates the question
    """
    template_name = 'qa/update_question.html'
    model = Question
    pk_url_kwarg = 'question_id'
    fields = ['title', 'description', 'tags']

    def get_success_url(self):
        question = self.get_object()
        return reverse('qa_detail', kwargs={'pk': question.pk})


class CreateAnswerView(LoginRequired, CreateView):
    """
    View to create new answers for a given question
    """
    template_name = 'qa/create_answer.html'
    model = Answer
    fields = ['answer_text']
    message = _('Thank you! your answer has been posted.')

    def form_valid(self, form):
        """
        Creates the required relationship between answer
        and user/question
        """
        form.instance.user = self.request.user
        form.instance.question_id = self.kwargs['question_id']
        return super(CreateAnswerView, self).form_valid(form)

    def get_success_url(self):
        if qa_messages:
            messages.success(
                self.request, self.message)
        return reverse('qa_detail', kwargs={'pk': self.kwargs['question_id']})


class UpdateAnswerView(LoginRequired, AuthorRequiredMixin, UpdateView):
    """
    Updates the question answer
    """
    template_name = 'qa/update_answer.html'
    model = Answer
    pk_url_kwarg = 'answer_id'
    fields = ['answer_text']

    def get_success_url(self):
        answer = self.get_object()
        return reverse('qa_detail', kwargs={'pk': answer.question.pk})


class CreateAnswerCommentView(LoginRequired, CreateView):
    """
    View to create new comments for a given answer
    """
    template_name = 'qa/create_comment.html'
    model = AnswerComment
    fields = ['comment_text']
    message = _('Thank you! your comment has been posted.')

    def form_valid(self, form):
        """
        Creates the required relationship between answer
        and user/comment
        """
        form.instance.user = self.request.user
        form.instance.answer_id = self.kwargs['answer_id']
        return super(CreateAnswerCommentView, self).form_valid(form)

    def get_success_url(self):
        if qa_messages:
            messages.success(
                self.request, self.message)
        question_pk = Answer.objects.get(
            id=self.kwargs['answer_id']).question.pk
        return reverse('qa_detail', kwargs={'pk': question_pk})


class CreateQuestionCommentView(LoginRequired, CreateView):
    """
    View to create new comments for a given question
    """
    template_name = 'qa/create_comment.html'
    model = QuestionComment
    fields = ['comment_text']
    message = _('Thank you! your comment has been posted.')

    def form_valid(self, form):
        """
        Creates the required relationship between question
        and user/comment
        """
        form.instance.user = self.request.user
        form.instance.question_id = self.kwargs['question_id']
        return super(CreateQuestionCommentView, self).form_valid(form)

    def get_success_url(self):
        if qa_messages:
            messages.success(
                self.request, self.message)
        return reverse('qa_detail', kwargs={'pk': self.kwargs['question_id']})


class UpdateQuestionCommentView(LoginRequired,
                                AuthorRequiredMixin, UpdateView):
    """
    Updates the comment question
    """
    template_name = 'qa/create_comment.html'
    model = QuestionComment
    pk_url_kwarg = 'comment_id'
    fields = ['comment_text']

    def get_success_url(self):
        question_comment = self.get_object()
        return reverse(
            'qa_detail', kwargs={'pk': question_comment.question.pk})


class UpdateAnswerCommentView(UpdateQuestionCommentView):
    """
    Updates the comment answer
    """
    model = AnswerComment

    def get_success_url(self):
        answer_comment = self.get_object()
        return reverse(
            'qa_detail', kwargs={'pk': answer_comment.answer.question.pk})


class QuestionDetailView(DetailView):
    """
    View to call a question and to render all the details about that question.
    """
    model = Question
    template_name = 'qa/detail_question.html'
    context_object_name = 'question'

    def get_context_data(self, **kwargs):
        answers = self.object.answer_set.all().order_by('pub_date')
        context = super(QuestionDetailView, self).get_context_data(**kwargs)
        context['last_comments'] = self.object.questioncomment_set.order_by(
            'pub_date')[:5]
        context['answers'] = list(answers.select_related('user')\
            .select_related('user__userqaprofile')\
            .annotate(answercomment_count=Count('answercomment')))
        return context

    def get(self, request, **kwargs):
        my_object = self.get_object()
        slug = kwargs.get('slug', '')
        if slug != slugify(my_object.title):
            kwargs['slug'] = slugify(my_object.title)
            return redirect(reverse('qa_detail', kwargs=kwargs))
        else:
            my_object.views += 1
            my_object.save()
            return super(QuestionDetailView, self).get(request, **kwargs)

    def get_object(self):
        # Call the superclass
        question = super(QuestionDetailView, self).get_object()
        return question


class ParentVoteView(View):
    """
    Base class to create a vote for a given model (question/answer)
    """
    model = None
    vote_model = None

    def get_vote_kwargs(self, user, vote_target):
        """
        This takes the user and the vote and adjusts the kwargs
        depending on the used model.
        """
        object_kwargs = {'user': user}
        if self.model == Question:
            target_key = 'question'
        elif self.model == Answer:
            target_key = 'answer'
        else:
            raise ValidationError('Not a valid model for votes')
        object_kwargs[target_key] = vote_target
        return object_kwargs

    def post(self, request, object_id):
        vote_target = get_object_or_404(self.model, pk=object_id)
        if vote_target.user == request.user:
            raise ValidationError(
                'Sorry, voting for your own answer is not possible.')
        else:
            upvote = request.POST.get('upvote', None) is not None
            object_kwargs = self.get_vote_kwargs(request.user, vote_target)
            vote, created = self.vote_model.objects.get_or_create(
                defaults={'value': upvote},
                **object_kwargs)
            if created:
                vote_target.user.userqaprofile.points += 1 if upvote else -1
                if upvote:
                    vote_target.positive_votes += 1
                else:
                    vote_target.negative_votes += 1
            else:
                if vote.value == upvote:
                    vote.delete()
                    vote_target.user.userqaprofile.points += -1 if upvote else 1
                    if upvote:
                        vote_target.positive_votes -= 1
                    else:
                        vote_target.negative_votes -= 1
                else:
                    vote_target.user.userqaprofile.points += 2 if upvote else -2
                    vote.value = upvote
                    vote.save()
                    if upvote:
                        vote_target.positive_votes += 1
                        vote_target.negative_votes -= 1
                    else:
                        vote_target.negative_votes += 1
                        vote_target.positive_votes -= 1

            vote_target.user.userqaprofile.save()
            if self.model == Question:
                vote_target.reward = question_score(vote_target)
            if self.model == Answer:
                vote_target.question.reward = question_score(
                    vote_target.question)
                vote_target.question.save()
            vote_target.save()

        next_url = request.POST.get('next', None)
        if next_url is not None:
            return redirect(next_url)
        else:
            return redirect(reverse('qa_index'))


class AnswerVoteView(ParentVoteView):
    """
    Class to upvote answers
    """
    model = Answer
    vote_model = AnswerVote


class QuestionVoteView(ParentVoteView):
    """
    Class to upvote questions
    """
    model = Question
    vote_model = QuestionVote


def profile(request, user_id):
    user_ob = get_user_model().objects.get(id=user_id)
    user = UserQAProfile.objects.get(user=user_ob)
    context = {'user': user}
    return render(request, 'qa/profile.html', context)



from django.conf.urls import url, include
from . import views


urlpatterns = [
    url(r'^$', views.QuestionIndexView.as_view(), name='qa_index'),

    url(r'^question/(?P<pk>\d+)/$',
        views.QuestionDetailView.as_view(), name='qa_detail'),

    url(r'^question/(?P<pk>\d+)/(?P<slug>[-_\w]+)$',
        views.QuestionDetailView.as_view(), name='qa_detail'),

    url(r'^question/answer/(?P<answer_id>\d+)/$',
        views.AnswerQuestionView.as_view(), name='qa_answer_question'),

    url(r'^question/close/(?P<question_id>\d+)/$',
        views.CloseQuestionView.as_view(), name='qa_close_question'),

    url(r'^new-question/$', views.CreateQuestionView.as_view(),
        name='qa_create_question'),

    url(r'^edit-question/(?P<question_id>\d+)/$',
        views.UpdateQuestionView.as_view(),
        name='qa_update_question'),

    url(r'^answer/(?P<question_id>\d+)/$',
        views.CreateAnswerView.as_view(), name='qa_create_answer'),

    url(r'^answer/edit/(?P<answer_id>\d+)/$',
        views.UpdateAnswerView.as_view(), name='qa_update_answer'),

    url(r'^vote/question/(?P<object_id>\d+)/$',
        views.QuestionVoteView.as_view(), name='qa_question_vote'),

    url(r'^vote/answer/(?P<object_id>\d+)/$',
        views.AnswerVoteView.as_view(), name='qa_answer_vote'),

    url(r'^comment-answer/(?P<answer_id>\d+)/$',
        views.CreateAnswerCommentView.as_view(),
        name='qa_create_answer_comment'),

    url(r'^comment-question/(?P<question_id>\d+)/$',
        views.CreateQuestionCommentView.as_view(),
        name='qa_create_question_comment'),

    url(r'^comment-question/edit/(?P<comment_id>\d+)/$',
        views.UpdateQuestionCommentView.as_view(),
        name='qa_update_question_comment'),

    url(r'^comment-answer/edit/(?P<comment_id>\d+)/$',
        views.UpdateAnswerCommentView.as_view(),
        name='qa_update_answer_comment'),

    url(r'^search/$', views.QuestionsSearchView.as_view(), name='qa_search'),

    url(r'^tag/(?P<tag>[-\w]+)/$',
        views.QuestionsByTagView.as_view(), name='qa_tag'),

    url(r'^profile/(?P<user_id>\d+)/$', views.profile, name='qa_profile'),

    url('^markdown/', include('django_markdown.urls')),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='answer',
            name='votes',
        ),
        migrations.RemoveField(
            model_name='question',
            name='votes',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0003_auto_20160414_1413'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='updated',
            field=models.DateTimeField(default=datetime.datetime(2016, 5, 5, 16, 11, 48, 760837, tzinfo=utc), verbose_name=b'date updated', auto_now=True),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0005_auto_20160519_1057'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='total_points',
            field=models.IntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-07-19 12:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qa', '0007_answer_total_points'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='userqaprofile',
            name='picture',
        ),
        migrations.AlterField(
            model_name='answer',
            name='updated',
            field=models.DateTimeField(auto_now=True, verbose_name='date updated'),
        ),
    ]



from django.test import TestCase
from django.contrib.auth import get_user_model
from django.utils import timezone

from qa.models import Question, Answer, AnswerComment, QuestionComment,\
    QuestionVote, AnswerVote, VoteParent


class BasicTaggingTest(object):
    """Defining a basic testcase method to be able to validate the tags on each
    model record, this is needed because the way django-taggit works.

    Disclaimer:
    Taken without remorse from:
    https://github.com/alex/django-taggit/blob/develop/tests/tests.py
    """
    def assert_tags_equal(self, qs, tags, sort=True, attr="name"):
        got = [getattr(obj, attr) for obj in qs]
        if sort:
            got.sort()
            tags.sort()
        self.assertEqual(got, tags)


class TestModels(TestCase, BasicTaggingTest):
    """TestCase class to test the models functionality
    """

    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username='test_user',
            email='test@swapps.co',
            password='top_secret'
        )
        self.other_user = get_user_model().objects.create_user(
            username='other_test_user',
            email='other_test@swapps.co',
            password='top_secret'
        )
        self.first_question = Question.objects.create(
            title="Another Question",
            description="A not so long random text to fill this field",
            pub_date=timezone.datetime(2016, 1, 6, 0, 0, 0),
            reward=0,
            views=3,
            user=self.user,
            closed=False,
        )
        self.first_answer = Answer.objects.create(
            question=self.first_question,
            answer_text="I hope this text is acceptable by django_markdown",
            pub_date=timezone.datetime(2016, 2, 6, 0, 0, 0),
            user=self.user,
        )

    def test_question(self):
        self.first_question.tags.add('one tag', 'the next tag', 'another tag')
        self.assertEqual(self.first_question.title, "Another Question")
        self.assertTrue(isinstance(self.first_question, Question))
        self.assertNotEqual(self.first_question.pub_date, timezone.now())
        self.assert_tags_equal(self.first_question.tags.all(),
                               ['one tag', 'the next tag', 'another tag'])
        self.assertEqual(self.first_question.views, 3)

    def test_answer(self):
        answer = Answer.objects.create(
            question=self.first_question,
            answer_text="A text body",
            pub_date=timezone.datetime(2016, 2, 7, 0, 0, 0),
            user=self.user,
        )
        self.assertTrue(isinstance(answer, Answer))
        self.assertTrue(isinstance(self.first_answer, Answer))
        self.assertEqual(self.first_answer.answer_text,
                         "I hope this text is acceptable by django_markdown")
        self.assertEqual(answer.answer_text, "A text body")

    def test_answer_comment(self):
        comment = AnswerComment.objects.create(
            answer=self.first_answer,
            comment_text="This is not so bright a comment",
            pub_date=timezone.datetime(2016, 2, 8, 0, 0, 0),
            user=self.user)
        self.assertTrue(isinstance(comment, AnswerComment))

    def test_question_comment(self):
        comment = QuestionComment.objects.create(
            question=self.first_question,
            comment_text="This is not so bright a comment",
            pub_date=timezone.datetime(2016, 2, 8, 0, 0, 0),
            user=self.user)
        self.assertTrue(isinstance(comment, QuestionComment))

    def test_question_vote(self):
        vote = QuestionVote.objects.create(user=self.user,
                                           value=True,
                                           question=self.first_question)
        self.assertTrue(isinstance(vote, QuestionVote))

    def test_answer_vote(self):
        vote = AnswerVote.objects.create(user=self.user,
                                         value=True,
                                         answer=self.first_answer)
        self.assertTrue(isinstance(vote, AnswerVote))

# this should be tested at the views, it is not a property of the model anymore

#    def test_question_positive_votes(self):
#        QuestionVote.objects.create(user=self.user,
#                                    value=True,  question=self.first_question)
#        self.assertEqual(self.first_question.positive_votes, 1)
#
#    def test_question_negative_votes(self):
#        QuestionVote.objects.create(user=self.user,
#                                    value=False,  question=self.first_question)
#        self.assertEqual(self.first_question.negative_votes, 1)
#
#    def test_question_total_points(self):
#        QuestionVote.objects.create(user=self.user,
#                                    value=True,  question=self.first_question)
#        QuestionVote.objects.create(user=self.other_user,
#                                    value=False, question=self.first_question)
#        self.assertEqual(self.first_question.total_points, 0)

#     def test_answer_positive_votes(self):
#         AnswerVote.objects.create(user=self.user,
#                                   value=True, answer=self.first_answer)
#         self.assertEqual(self.first_answer.positive_votes, 1)
# 
#     def test_answer_negative_votes(self):
#         AnswerVote.objects.create(user=self.user,
#                                   value=False, answer=self.first_answer)
#         self.assertEqual(self.first_answer.negative_votes, 1)
# 
#     def test_answer_total_points(self):
#         AnswerVote.objects.create(user=self.user,
#                                   value=True, answer=self.first_answer)
#         AnswerVote.objects.create(user=self.other_user,
#                                   value=False, answer=self.first_answer)
#         self.assertEqual(self.first_answer.total_points, 0)
